import logging
import uuid

from src.api.services.recognition_track import RecognitionTrack
from src.config import Config

from fastapi import APIRouter, Request, HTTPException
from fastapi.responses import HTMLResponse, JSONResponse
from fastapi.templating import Jinja2Templates

from aiortc import RTCSessionDescription, RTCPeerConnection

from src.schemas.schemas import WatchData

logger = logging.getLogger(__name__)

identification_router = APIRouter(prefix='/identification', tags=['Identification'])
conf = Config()
templates = Jinja2Templates(directory='templates')
pcs = set()


@identification_router.get('/', response_class=HTMLResponse)
async def identification(request: Request):
    ctx = {'methods': conf.get_methods(only_names=True, methods_names=['Identification'])}
    return templates.TemplateResponse(request=request, name='identification_v2.html', context=ctx)


@identification_router.post('/watch/', response_class=JSONResponse)
async def watch(watch_data: WatchData):
    if not watch_data.task:
        raise HTTPException(status_code=400, detail='You select "Disable"')

    offer = RTCSessionDescription(sdp=watch_data.sdp, type=watch_data.type)

    pc = RTCPeerConnection()
    pc_id = "PeerConnection(%s)" % uuid.uuid4()
    pcs.add(pc)

    def log_info(msg, *args):
        logger.info(pc_id + ' ' + msg, *args)

    log_info('Created')

    @pc.on('iceconnectionstatechange')
    async def on_iceconnectionstatechange():
        print("ICE connection state is %s", pc.iceConnectionState)
        if pc.iceConnectionState == "failed":
            await pc.close()
            pcs.discard(pc)

    @pc.on('track')
    def on_track(track):
        log_info(f'Track {track.kind} received')

        data_channel = pc.createDataChannel('data', maxRetransmits=0, ordered=False)
        local_video = RecognitionTrack(track, data_channel, task=watch_data.task)
        pc.addTrack(local_video)

        @track.on('ended')
        async def on_ended():
            log_info(f'Track {track.kind} ended')
            local_video.finish()
            del local_video

    # handle offer
    await pc.setRemoteDescription(offer)

    # send answer
    answer = await pc.createAnswer()
    await pc.setLocalDescription(answer)

    return {'sdp': pc.localDescription.sdp, 'type': pc.localDescription.type}
