import logging
import time

import numpy as np

from src.db.main import InterfaceDB
from src.models.main import InterfaceModel
from src.utils import decode_image, encode_image

logger = logging.getLogger(__name__)


class TaskManager:
    def __init__(self):
        self.__byte_data = None
        self.__predictions = None

    def get_predictions_names(self):
        predictions_names = dict()
        try:
            for method_model in self.__predictions:
                method, model = method_model.split("_")
                if method in predictions_names:
                    predictions_names[method].append(model)
                else:
                    predictions_names[method] = [model]

        except Exception as error:
            print("An exception occurred (web/manager):", error)

        return predictions_names

    def get_presaved_data(self, methods_models: list[str] = None) -> dict:
        if methods_models:
            predictions = dict()
            for method_model in methods_models:
                predictions[method_model] = self.__predictions[method_model]
        else:
            predictions = self.__predictions

        return dict(
            byte_data=self.__byte_data,
            predictions=predictions
        )

    def get_saved_data(self, command: str, n: int, data_base: InterfaceDB) -> dict:
        saved_data = dict()
        try:
            images = data_base.get("images")
            data = data_base.get(command)
            for row_id, values in data.items():
                if n == 0:
                    break

                saved_data[row_id] = values
                saved_data[row_id].update(images[f"image_{values['ts']}"])

                n -= 1

        except Exception as error:
            print("An exception occurred (web/manager):", error)

        return saved_data

    def execute_commands(self, data: dict, models: dict[str, InterfaceModel],
                         data_base: InterfaceDB = None) -> tuple[np.ndarray, dict]:
        b_image: np.ndarray = data["byte_data"]
        predictions = dict()
        try:
            # image = decode_image(b_image)
            canvas = b_image.copy()
            for com in data["commands"]:
                model = models[com]
                predictions[com] = model.predict(b_image, {"data_base": data_base})
                canvas = model.apply_predictions(canvas, predictions[com])

            self.__byte_data = data["byte_data"]
            self.__predictions = predictions
            b_image = canvas
        except Exception as error:
            logger.error(f'An exception occurred (web/manager): {error}')

        return b_image, predictions

    def draw_predictions(self,
                         commands: list[str],
                         models: dict[str, InterfaceModel],
                         frame: np.ndarray,
                         predictions: dict[str, dict]):

        canvas = frame.copy()

        for com in commands:
            model = models[com]
            canvas = model.apply_predictions(canvas, predictions[com])

        return canvas

    def execute_save(self, data: dict, data_base: InterfaceDB) -> dict:
        status = dict()
        try:
            # Define timestamp and image name
            ts = time.time()
            img_name = f"image_{ts}"
            # Save images
            img_data = {img_name: dict(
                ts=ts,
                byte_image=self.__byte_data
            )}
            status["images"] = data_base.add("images", img_data)
            # Start to save methods data
            for com in data["commands"]:
                com_data = {img_name: dict(ts=ts)}
                if "changed_data" in data["params"]:
                    com_data[img_name].update(data["params"]["changed_data"][com])
                else:
                    com_data[img_name].update(self.__predictions[com])

                status[com] = data_base.add(com, com_data)
                # Save data for identification method
                if len(data["params"]["anns"]):
                    status[f"{com}_anns"] = self.__save_identification_annotations(data["params"], com, img_name, ts, data_base)

        except Exception as error:
            print("An exception occurred (web/manager):", error)

        return status

    def __save_identification_annotations(self, params, command, img_name, ts, data_base) -> bool:
        status = False
        if params["anns"][command]:
            if "changed_data" in params:
                data = params["changed_data"][command]
            else:
                data = self.__predictions[command]

            for i, person_name in enumerate(data["name"]):
                if "Unknown_person" not in person_name:
                    person_data = {f"{person_name}_{ts}": dict(
                        image_name=img_name,
                        ts=ts,
                        person_name=person_name,
                        d=data["d"][i],
                        bbox=data["bbox"][i],
                        embedding=data["embedding"][i]
                    )}
                    status = data_base.add(command + "_anns", person_data)

        return status

    def execute_delete(self, data: dict, data_base: InterfaceDB) -> bool:
        command = data["commands"][0]
        status = data_base.delete(command, data["params"]["image_id"])
        return status
