import logging
from fastapi import FastAPI, Request
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from fastapi.responses import HTMLResponse, JSONResponse
from fastapi.middleware.cors import CORSMiddleware

from src.core.manager import TaskManager
from src.schemas.schemas import FrontedData
from src.config import Config

logger = logging.getLogger(__name__)

app = FastAPI()
app.mount('/static', StaticFiles(directory="static"), name="static")
templates = Jinja2Templates(directory="templates")

conf = Config()
task_manager = TaskManager()

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


def init_routes():
    from src.api.endpoints.identification import identification_router
    from src.api.endpoints.collection import collection_router

    app.include_router(identification_router)
    app.include_router(collection_router)


# Main page
@app.get("/", response_class=HTMLResponse)
def welcome(request: Request):
    return templates.TemplateResponse(
        request=request, name="welcome.html"
    )


# Record page
@app.get("/record", response_class=HTMLResponse)
def record(request: Request):
    ctx = {"methods": conf.get_methods(only_names=True)}
    return templates.TemplateResponse(
        request=request, name="record.html", context=ctx
    )


@app.post("/predict", response_class=JSONResponse)
async def predict(frontend_data: FrontedData):
    response = None
    b_data = None
    try:
        b_data, response = task_manager.execute_commands(frontend_data.model_dump(), conf.get_methods(), conf.data_base)
    except Exception as error:
        logger.error("An exception occurred (web/main):", error)

    return {"byte_image": b_data, "predictions": response}


# Save page
@app.get("/presave/{prev_page}", response_class=HTMLResponse)
async def presave(prev_page: str, request: Request):
    ctx = {"methods_models": task_manager.get_predictions_names()}
    return templates.TemplateResponse(
        request=request, name=f"presave_{prev_page}.html", context=ctx
    )


# new
@app.post("/get_presaved_data", response_class=JSONResponse)
async def get_presaved_data(fronted_data: FrontedData = None):
    presaved_data = None
    try:
        predictions_names = fronted_data.commands if fronted_data else None
        presaved_data = task_manager.get_presaved_data(methods_models=predictions_names)
    except Exception as error:
        print("An exception occurred:", error)

    return presaved_data


@app.post("/save", response_class=JSONResponse)
async def save(frontend_data: FrontedData):
    status = None
    try:
        if len(frontend_data.commands):
            status = task_manager.execute_save(frontend_data.model_dump(), conf.data_base)
    except Exception as error:
        print("An exception occurred:", error)

    return {"status": status}


# Delete data
@app.post("/delete", response_class=JSONResponse)
async def delete(fronted_data: FrontedData):
    status = None
    try:
        status = task_manager.execute_delete(fronted_data.model_dump(), conf.data_base)
    except Exception as error:
        print("An exception occurred:", error)

    return {"response": status}


init_routes()
