import numpy as np
import cv2
import face_recognition
from sklearn.metrics import pairwise

from src.db.main import InterfaceDB
from src.models.main import InterfaceModel


class IdentificationFaceRecognitionModel(InterfaceModel):
    def __init__(self, std=0.015):
        self.std = std

    def predict(self, decoded_data: np.array, params: dict) -> dict:
        results = dict(
            name=[],
            d=[],
            bbox=[],
            embedding=[]
        )
        try:
            faces_locations = face_recognition.face_locations(decoded_data)
            for i, face_locations in enumerate(faces_locations):
                embedding = face_recognition.face_encodings(decoded_data, [face_locations])[0]
                d = pairwise.cosine_distances(embedding[None, :], np.ones(128)[None, :])[0, 0]
                sim_refs = self.__get_similar_references(d, params['data_base'])

                person_name = self.__identify_person(embedding, sim_refs)
                if person_name is None:
                    person_name = f"Unknown_person_{i + 1}"

                results["name"] += [person_name]
                results["d"] += [d]
                results["bbox"] += [face_locations]
                results["embedding"] += [list(embedding)]

        except Exception as error:
            print("An exception occurred (models/Identification_FaceRecognition):", error)

        return results

    def apply_predictions(self, image: np.array, predictions: dict) -> np.array:
        """Apply predictions to the data stream"""
        for [y_top, x_right, y_bottom, x_left], name in zip(predictions["bbox"], predictions["name"]):
            thickness = 2
            if "Unknown_person" in name:
                color = (255, 0, 0)
            else:
                color = (0, 255, 0)

            cv2.rectangle(image, (x_left, y_top), (x_right, y_bottom), color, thickness)
            cv2.putText(image, name, (x_left, y_bottom + 25), cv2.FONT_HERSHEY_SIMPLEX,
                        1, color, thickness, cv2.LINE_AA)

        return image

    def __identify_person(self, embedding: list[float], similar_references: dict[str, list], thresholder=0.6, step=0.05):
        refs_ids = list(similar_references.keys())
        refs_embeddings = list(similar_references.values())

        results = face_recognition.compare_faces(refs_embeddings, embedding, tolerance=thresholder)

        suspects_ids = np.where(results)[0]
        if len(suspects_ids) == 0:
            return None

        suspects = {refs_ids[i]: refs_embeddings[i] for i in suspects_ids}

        suspects_names = np.unique(["_".join(ref_id.split('_')[:-1]) for ref_id in suspects])
        if len(suspects_names) == 1:
            return suspects_names[0]

        return self.__identify_person(embedding, suspects, thresholder - step, step)

    def __get_similar_references(self, d: float, data_base: InterfaceDB) -> dict:
        sim_refs = dict()

        left = d - self.std
        right = d + self.std

        anns = data_base.get('Identification_FaceRecognition_anns', ["d", "embedding"])
        for person_id, data in anns.items():
            if left <= data["d"] <= right:
                sim_refs[person_id] = data["embedding"]

        return sim_refs
